@@ -1,6 +1,6 @@
 package Math::Vector::Real;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use strict;
 use warnings;
@@ -170,7 +170,7 @@ sub div {
 sub div_me {
     croak "can't use vector as dividend" if ref $_[1];
     my $v = shift;
-    my $i = 1 / shift;
+    my $i = 1.0 / shift;
     $_ *= $i for @$v;
     $v;
 }
@@ -336,6 +336,16 @@ sub first_orthant_reflection {
     bless [map CORE::abs, @$self];
 }
 
+sub sum {
+    ref $_[0] or shift; # works both as a class and as an instance method
+    my $sum;
+    if (@_) {
+        $sum = V(@{shift()});
+        $sum += $_ for @_;
+    }
+    return $sum;
+}
+
 sub box {
     shift;
     return unless @_;
@@ -370,22 +380,70 @@ sub nearest_in_box {
     $p
 }
 
-sub nearest_in_box_border {
-    my $p = shift->clone;
-    my ($b0, $b1) = Math::Vector::Real->box(@_);
-    my ($min_d, $comp, $comp_ix);
-    for my $q ($b0, $b1) {
-        for (0..$#$p) {
-            my $d = CORE::abs($p->[$_] - $q->[$_]);
-            if (!defined $min_d or $min_d > $d) {
-                $min_d = $d;
-                $comp = $q->[$_];
-                $comp_ix = $_;
-            }
+sub dist2_to_box {
+    @_ > 1 or croak 'Usage: $v->dist2_to_box($w0, ...)';
+    my $p = shift;
+    my $d2 = 0;
+    my ($min, $max) = Math::Vector::Real->box(@_);
+    for (0..$#$p) {
+        if ($p->[$_] < $min->[$_]) {
+            my $d = $p->[$_] - $min->[$_];
+            $d2 += $d * $d;
+        }
+        elsif ($p->[$_] > $max->[$_]) {
+            my $d = $p->[$_] - $max->[$_];
+            $d2 += $d * $d;
         }
     }
-    $p->[$comp_ix] = $comp;
-    wantarray ? ($p, $min_d) : $p;
+    $d2;
+}
+
+sub nearest_in_box_border {
+    # TODO: this method can be optimized
+    my $p = shift->clone;
+    my ($b0, $b1) = Math::Vector::Real->box(@_);
+    my $in = 0;
+    for (0..$#$p) {
+        if ($p->[$_] < $b0->[$_]) {
+            $p->[$_] = $b0->[$_];
+        }
+        elsif ($p->[$_] > $b1->[$_]) {
+            $p->[$_] = $b1->[$_];
+        }
+        else {
+            $in++;
+        }
+    }
+    if ($in == @$p) {
+        # vector was inside the box
+        my $min_d = 'inf';
+        my ($comp, $comp_ix);
+        for my $q ($b0, $b1) {
+            for (0..$#$p) {
+                my $d = CORE::abs($p->[$_] - $q->[$_]);
+                if ($min_d > $d) {
+                    $min_d = $d;
+                    $comp = $q->[$_];
+                    $comp_ix = $_;
+                }
+            }
+        }
+        $p->[$comp_ix] = $comp;
+    }
+    $p;
+}
+
+sub max_dist2_to_box {
+    @_ > 1 or croak 'Usage: $v->max_dist2_to_box($w0, ...)';
+    my $p = shift;
+    my ($c0, $c1) = Math::Vector::Real->box(@_);
+    my $d2 = 0;
+    for (0..$#$p) {
+        my $d0 = CORE::abs($c0->[$_] - $p->[$_]);
+        my $d1 = CORE::abs($c1->[$_] - $p->[$_]);
+        $d2 += ($d0 >= $d1 ? $d0 * $d0 : $d1 * $d1);
+    }
+    return $d2;
 }
 
 sub max_dist2_between_boxes {
@@ -725,6 +783,18 @@ used to find the point nearest to C<$v> from inside the rectangle:
 Note that if C<$v> lays inside the box, the nearest point is C<$v>
 itself. Otherwise it will be a point from the box hyper-surface.
 
+=item $d2 = $v->dist2_to_box($w0, $w1, ...)
+
+Calculates the square of the minimal distance between the vector C<$v>
+and the minimal axis-aligned box containing all the vectors C<($w0,
+$w1, ...)>.
+
+=item $d2 = $v->max_dist2_to_box($w0, $w1, ...)
+
+Calculates the square of the maximum distance between the vector C<$v>
+and the minimal axis-aligned box containing all the vectors C<($w0,
+$w1, ...)>.
+
 =item $d2 = Math::Vector::Real->max_dist2_between_boxes($a0, $a1, $b0, $b1)
 
 Returns the square of the maximum distance between any two points
@@ -758,6 +828,10 @@ and another normal.
 
 In scalar context returns the normal vector.
 
+=item $v = Math::Vector::Real->sum(@v)
+
+Returns the sum of all the given vectors.
+
 =item @b = Math::Vector::Real->complementary_base(@v)
 
 Returns a base for the subspace complementary to the one defined by
